      Program F96_2SMC

* Reads files in Herrmann's File96 format and reformats them into SMC format.

* This version assumes that the time series are 3 component time series
* (resulting from Mech96)... it checks the first line for this.

* The names of the files are read in from a file specified at run time.
* The output file names are constructed by appending the first 7 characters
* of the input file name with the component designator found in the File96
* file and an extension of "smc".


* Here is a sample input file:

*Input for F96_2SMC   ! This is line 1 of the sample; column 1 of the sample
*Factor:              ! follows the asterisk (*)
* 1.0e+24
*File in F96 format; stem for output (max of 7 chars, to leave room for compnt):
*xsoirs2.out         xsoirs2
*xsoiss2.out         xsoiss2
*soilrs2.out         soilrs2
*soilss2.out         soilss2
*watrrs2.out         watrrs2
*watrss2.out         watrss2
* stop        ! this must start in column 2* Factor:

* Dates: 12/15/97 - Written by D.M. Boore
*        12/20/97 - improved input file structure
*        12/04/00 - added ibgn, iend to rcf, rci, rcc calling arguments

      real acc(4100)
      character f_in*40, f_96*40, f_out*40, buff*40, buffer*80,
     :          stem*40
      logical f_in_exist


* Get the name of the file with file names:


      f_in_exist = .false.
      do while (.not. f_in_exist)
        f_in = ' '
        write(*, '(a\)') 
     :    ' Enter name of file with filenames (cr to quit): '
        read(*, '(a)') f_in
        if (f_in(1:4) .eq. '    ') stop
        inquire(file=f_in, exist=f_in_exist)
        if (.not. f_in_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      nu_in = 10
      open(unit=nu_in, file=f_in, status='unknown')

* Read a common multiplicative factor (this is needed because the SMC
* format allows only 1 digit for the power of ten):

      call skip(nu_in, 2)
      read(nu_in, *) factor
      write(*,'(a,1x,1pe10.3)') ' Factor = ', factor
      call skip(nu_in, 1)

999   continue

      buffer = ' '
      read(nu_in, '(a)') buffer
      call upstr(buffer)
      if (buffer(2:5) .eq. 'STOP') then
        close(unit = nu_in)
        stop
      end if

      f_96 = ' '
      call rcc(1, buffer, 80, f_96, nc_f96,ibgn,iend)

      stem = ' '
      call rcc(2, buffer, 80, stem, nc_stem,ibgn,iend)
      if (nc_stem .gt. 7) then
        write(*, '(1x,a,1x,i2,1x,a)')
     :        ' ** ERROR **, nc_stem = ', nc_stem, ', > 7-- QUITTING!'
        stop
      end if

      nu_f96 = 20
      open(unit=nu_f96, file= f_96, status='unknown')

      buff = ' ' 
      read(nu_f96, '(a)') buff

      call upstr(buff)
      if (buff(1:6) .ne. 'FILE03') then
        write(*, '(3a)') ' buff(1:6) = ', buff(1:6), 
     :  ' .ne. FILE03; quitting!'
        close(unit=nu_f96)
        close(unit=nu_in)
        stop
      end if

      call skip(nu_f96, 4)
 
      read(nu_f96, *) d1,d2,d3,d4,d5,d6,d7,d8, source_depth
      call skip(nu_f96, 1)
      read(nu_f96, *) d1,d2, sta_elev
      read(nu_f96, *) d_km, d_deg, az, baz

      call skip(nu_f96, 8)

      do i = 1, 3         ! Loop over components

        buff = ' '
        read(nu_f96, '(a)') buff

* construct output file name:

        f_out = ' '
        f_out = stem(1:nc_stem)//buff(1:1)//'.smc'

        read(nu_f96, *) v_orient, h_orient, dt, npts
        ivert = int(v_orient)
        ihor = int(h_orient)

        read(nu_f96, *) iyr, imon, iday, ihr, imin, sec
        isec = int(sec)
        imsec = int(1000.0*(sec-isec))
     
        read(nu_f96, 444) (acc(j), j=1, npts)        
444     format(e16.8,1x,e16.8,1x,e16.8,1x,e16.8)
        do j = 1, npts
          acc(j) = factor*acc(j)
        end do

* Now write out the data:

        tstart = 0.0
        tstop = float(npts-1)*dt
        call WriteSMC(f_out, acc, tstart, tstop,
     :  dt, npts, ivert, ihor,
     :  d_km, az, source_depth, sta_elev,
     :  iyr, imon, iday, ihr, imin, isec, imsec)          
 
      end do

      go to 999
888   continue

      stop
      end

* ---------------------- BEGIN SKIP -------------------
      subroutine SKIP(lunit, nlines)
        do i = 1, nlines
           read(lunit, *)
        end do
        return
      end
* ---------------------- END SKIP -------------------

* ---------------------- BEGIN DOY -------------------
      subroutine DOY (IYR4,ICMO,ICDY,IDOY,ISTAT)

c DOY - [MUELLER.FS.GEN]CSMGENLB
c Convert 4-digit year, calendar month and calendar day
c  (IYR4, ICMO, and ICDY) to day of year (IDOY).
c Valid for 1901=<IYR4=<2099 (simple leap-year test for this range).
c ISTAT= 0 for success;
c      = 1 if IYR4 is out-of-range;
c      = 2 if ICMO is out-of-range;
c      = 3 if ICDY is out-of-range.
c      If ISTAT>0, return with IDOY=0.
c Written by C.Mueller, USGS.

      dimension MDAYS(12)

      ISTAT = 0
      IDOY = 0
      if (IYR4.lt.1901.or.IYR4.gt.2099) then
        ISTAT = 1
        return
      endif
      ILEAP = 0
      if (MOD(IYR4,4).eq.0) ILEAP = 1
      MDAYS(1) = 31
      MDAYS(2) = 28+ILEAP
      MDAYS(3) = 31
      MDAYS(4) = 30
      MDAYS(5) = 31
      MDAYS(6) = 30
      MDAYS(7) = 31
      MDAYS(8) = 31
      MDAYS(9) = 30
      MDAYS(10) = 31
      MDAYS(11) = 30
      MDAYS(12) = 31
      if (ICMO.lt.1.or.ICMO.gt.12) then
        ISTAT = 2
        return
      endif
      if (ICDY.lt.1.or.ICDY.gt.MDAYS(ICMO)) then
        ISTAT = 3
        return
      endif
      IDOY = ICDY
      do 1 I=1,ICMO-1
1        IDOY = IDOY+MDAYS(I)
      return

      end
* ---------------------- END DOY -------------------

* ---------------------- BEGIN WriteSMC ---------------------
      subroutine WriteSMC(fnameout, accel, tstart, tstop,
     :  dt, naccel, ivert, ihor,
     :  d_km, az, source_depth, sta_elev,
     :  iyr, imon, iday, ihr, imin, isec, imsec)          

*  Reformats a time series 
*  into SMC format (the format used on the CD-ROM)

* Dates: 12/15/97 - revised, adding more input parameters

      real  rnull, accel(*), rhead(50)

      integer ncomments, naccel, inull, ihead(48)

      character fnameout*12

* Open output file:
      open(unit=30, file=fnameout, status='unknown')

* Write name to screen:
      write(*, '(2a)') ' Output file name: ', fnameout

* Write headers:

* First write the 11 lines of comments:

*      write(30, '(a)') '0 UNKNOWN'
      write(30, '(a)') '*'
      write(30, '(a)') '*'
      write(30, '(a)') ' stacode'
      write(30, '(a)') ' date, eqname'
      write(30, '(a)') 'Moment Mag='
      write(30, '(a,t30,a,i3,1x,a,i3)') 'station = ',  
     :                            'orient_h= ', ihor,
     :                            'orient_v= ', ivert
      write(30, '(a,t34,a)') 'epicentral dist =','pk acc ='
      write(30, '(a, t22, a)') 'inst type=DSA', 'data source = SCE'
      write(30, '(a)') '*'
      write(30, '(a)') '*'
      write(30, '(a)') '*'
          
* Figure out indices for the output window:

      indxstart = tstart/dt + 1
      indxstop = tstop/dt + 1
      if (indxstop .gt. naccel) indxstop = naccel

      nout = indxstop - indxstart + 1


* Now write the integer header block:
* First fill with null values:

      inull = -32768
      do i = 1, 48
        ihead(i) = inull
      end do

      call DOY (iyr, imon, iday, idoy, istat)

      ihead(1) = inull
      ihead(2) = iyr
      ihead(3) = idoy
      ihead(4) = ihr
      ihead(5) = imin
      ihead(6) = isec
      ihead(7) = imsec
      ihead(11) = 3
      ihead(13) = ivert
      ihead(14) = ihor
      ncomments = 0
      ihead(16) = ncomments
      ihead(17) = nout

      write(30, '(8I10)') (ihead(i), i=1,48)

* Now write the real header block:
* First fill with null values:
      rnull = 1.7e+38
      do i = 1, 50
        rhead(i) = rnull
      end do

      rhead(1) = rnull
      sps = 1.0/dt
      rhead(2) = sps
      rhead(5) = source_depth
      rhead(13) = sta_elev
      rhead(17) = d_km
      rhead(18) = az

      write(30, '(5E15.7)') (rhead(i), i=1,50)

* Write the acceleration values

      write(30, '(8(1pe10.4e1))') 
     :        (accel(j), j = indxstart, indxstop)
         
* That should be it; close the output file and loop back for another component

      close(unit=30)

      return
      end
* ---------------------- END WriteSMC ---------------------

* --------------------- BEGIN UPSTR ----------------------------------
      Subroutine UPSTR ( text )
* Converts character string in TEXT to uppercase
* Dates: 03/12/96 - Written by Larry Baker

C
      Implicit   None
C
      Character  text*(*)
C
      Integer    j
      Character  ch
C
      Do 1000 j = 1,LEN(text)
         ch = text(j:j)
         If ( LGE(ch,'a') .and. LLE(ch,'z') ) Then
            text(j:j) = CHAR ( ICHAR(ch) - ICHAR('a') + ICHAR('A') )
         End If
 1000    Continue
C
      Return
      End
* --------------------- END UPSTR ----------------------------------

* ----------------------- BEGIN RCC -----------------------------------
      subroutine RCC (KPARM,CSTR,NCSTR,C,NC)

c RCC - [MUELLER.FS.GEN]CSMGENLB
c Read character string C(1:NC), the KPARMth element in CSTR(1:NCSTR).
c  CSTR contains one-or-more mixed-type fields
c  separated by blanks or commas to the right of an optional '='.
c  KPARMth field contains C to the right of an optional '>'.
c If the KPARMth field is nonexistent or empty, RCC sets NC=0 and returns.
c  This simulates a <carriage-return> in a Q-format read.
c  (An empty field must be separated by commas, not blanks.)
c Written by C.Mueller, USGS.

      character CSTR*(*),C*(*)

c Eliminate trailing white space.
      N = NCSTR+1
1     N = N-1
      if (CSTR(N:N).eq.' '.or.CSTR(N:N).eq.',') goto 1
c Find optional '='.
      I2 = INDEX(CSTR(1:N),'=')
c Find start and end of KPARMth field.
      do 4 K=1,KPARM
         I1 = I2
2        I2 = I1
3        I1 = I1+1
         if (CSTR(I1:I1).eq.' ') goto 3
         if (CSTR(I2:I2).eq.' '.and.CSTR(I1:I1).eq.',') goto 2
         IBLANK = INDEX(CSTR(I1:N),' ')
         ICOMMA = INDEX(CSTR(I1:N),',')
         if (IBLANK.eq.0.and.ICOMMA.eq.0.and.K.lt.KPARM) goto 801
         if (IBLANK.eq.0) IBLANK = N-I1+2
         if (ICOMMA.eq.0) ICOMMA = N-I1+2
         I2 = I1+MIN(IBLANK,ICOMMA)-1
4        continue
c Check for empty field.
      if (I2.eq.I1) goto 801
c Find optional '>'.
      I1 = I1+INDEX(CSTR(I1:I2),'>')
c Get C.
      C = CSTR(I1:I2-1)
      NC = I2-I1
      return
c Errors.
801   NC = 0
      return

      end
* ----------------------- END RCC -----------------------------------
