
      Program SMC_MAKE

! Make a spike, pulse, step, ramp, ncycles of sine, or noise in smc format

!Input from a control file.  Here is an example:

!!Control file for program SMC_Make
!!Revision of program involving a change in the control file on this date:
!   12/04/19
!!As many comment lines as desired, each starting with "!"
!!
!!Type of signal and parameters for that type:
!!  1: spike or box  
!!    amp, tstart, tstop, tlength, sps
!!    (tstop=tstart for spike)
!!  2: step or ramp  
!!    amp, tstart, tstop, tlength, sps
!!    (tstop=tstart for step)
!!  3: sine
!!    amp, tstart, tlength, sps, period, ncycles, phase_strt
!!    (phase_strt is the fraction of period to start the sine)
!!  4: noise 
!!    amp, tstart, tstop, tlength, sps, Gaussian (0) or uniform (1), seed, n_noise_grams
!!
!!Enter type of signal, followed by the parameters for that type and filename
!!
! 1
! 20 10 10 50 100  ! spike or box: amp, tstart, tstop, tlength, sps
! spike
!! 2
!! 20 10 30 50 100 ! step or ramp: amp, tstart, tstop, tlength, sps
!! ramp
!! 3
!! 20 10 50 100 2 5 0  ! sine: amp, tstart, tlength, sps, period, ncycles, phase_strt 
!! sine
!! 4
!! 20 10 30 50 100, 0, 309, 4 ! noise: amp, tstart, tstop, tlength, sps, Gaussian (0) or uniform (1), seed, n_noise_grams
!! noise
!!
!!Enter precision of smc file (high=Y, otherwise normal)
! Y

! Dates: 12/16/01 - Written by D. Boore
!        07/14/02 - Added sin
!        07/25/02 - Added noise (a bunch, if desired)
!        07/26/02 - Force noise segment to have zero mean
!        08/02/02 - Do not force noise segment to have zero mean
!        11/21/03 - Change to be able to use lf95 compiler
!        11/22/03 - Modify n cycles of sin to start at a specified fraction
!                   of the period.
!        11/22/03 - Integral of sine was not correct, so I was more careful in
!                   defining nstop.
!        10/17/07 - Transparent change to user needed because smcread can
!                   read two input formats
!        01/21/08 - Add comment that option 1 can yield a box
!        01/19/10 - Increased amp field in comments
!        03/26/10 - Doubled amplitude of uniform white noise (ran1 returns
!                   0 to 1 and want a random number uniformly distributed
!                   between -1 and 1)
!        05/09/11 - Assign precision='HIGH'
!                 - Correct error in setting nstop for sine
!        06/04/11 - Ask for precision
!        11/07/19 - Get params from a control file
!        12/04/19 - Replace i_spike_step with i_type
!                 - Continue debugging

      character date_ctl_correct*8, date_ctl_in*30
      character f_ctl*60
      character cmnts2skip(500)*200
     

      character precision*8
      

      character char_head(11)*80, comments(50)*80, high_precision*5 
      integer int_head(48)
      real real_head(50)
      character f_smc*200, stem*200, c_count*3
      real y(:)  
      allocatable :: y
      
      logical f_ctl_exist

      twopi = 2.0*4.0*atan(1.0)

      inull = -32768
      rnull = 1.7e+38

      do i = 1, 48
        int_head(i) = inull
      end do
    
      do i = 1, 50
        real_head(i) = rnull
      end do
    
      do i = 1, 11
        char_head(i) = '*'
      end do

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=SMC_Make.CTL;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) == '    ') f_ctl = 'SMC_Make.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '12/04/19'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      if (date_ctl_correct(1:nc_date_ctl_correct) .ne. 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; '//
     :       'the proper date is '//
     :        date_ctl_correct(1:nc_date_ctl_correct)
        write(*,'(a)') 'Update your control file and rerun the program!'
        close(nu_ctl)
        stop
      end if
      
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)

      read(nu_ctl,*) i_type

      amp = 0.0
      tstart = 0.0
      tstop = 0.0
      tlength = 0.0
      sps = 0.0 
      period = 0.0
      ancycles = 0.0
      phase_strt = 0.0
      iran_type = 0
      seed = 0.0
      n_ts = 0
      
      
      SELECT CASE(i_type)
      CASE(1)
        read(nu_ctl,*) amp, tstart, tstop, tlength, sps
          nstart = tstart * sps + 1
          nstop = tstop * sps + 1
        stem = ' '
        read(nu_ctl,'(a)') stem
        call trim_c(stem, nc_stem)
      CASE(2)
        read(nu_ctl,*) amp, tstart, tstop, tlength, sps
          nstart = tstart * sps + 1
          nstop = tstop * sps + 1
        stem = ' '
        read(nu_ctl,'(a)') stem
        call trim_c(stem, nc_stem)
      CASE(3)
        read(nu_ctl,*) amp, tstart, tlength, sps, 
     :                 period, ancycles, phase_strt
          nstart = tstart * sps + 1
          nstop = nstart + (ancycles*period) * sps         ! 05/09/11
          freq = 1.0/period
        stem = ' '
        read(nu_ctl,'(a)') stem
        call trim_c(stem, nc_stem)
      CASE(4)
        read(nu_ctl,*) amp, tstart, tstop, tlength, sps, 
     :                 iran_type, seed, n_ts
          nstart = tstart * sps + 1
          nstop = tstop * sps + 1
        stem = ' '
        read(nu_ctl,'(a)') stem
        call trim_c(stem, nc_stem)
      CASE DEFAULT
        write(*,'(a,1x,i5,a)') 
     :      '  ERROR!!!, i_type = ', i_type,
     :      ' .ne. 1, 2, 3, or 4, QUITTING!'
        stop
      END SELECT
        
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)

      high_precision = ' '
      read(nu_ctl,*) high_precision
      call trim_c(high_precision, nc_high_precision)
      call upstr(high_precision)
      if (high_precision(1:1) == 'Y') then
        precision = ' '
        precision = 'HIGH'
        int_head(47) = 8
      else
        precision = ' '
        precision = 'STANDARD'
        int_head(47) = inull
      end if
       
      if(tlength < tstop) tlength = tstop
      
      npts = tlength * sps + 1

      write(*,'(a)') ' nstart, nstop, npts = '
      write(*,*) nstart, nstop, npts

      int_head(17) = npts
      real_head(2) = sps
      write(*,'(a)') ' int_head(17), real_head(2) = '
      write(*,*)       int_head(17), real_head(2)

 
      ncomments = 1
      comments(ncomments) = '|'
      ncomments = ncomments + 1
      comments(ncomments) = '|  File created with program SMC_MAKE'

      SELECT CASE(i_type)
      CASE(1)
        allocate(y(npts+2))
        y = 0.0
        do i = nstart, nstop
           y(i) = amp
        end do
        if (nstart .eq. nstop) then
          ncomments = ncomments + 1
          write(comments(ncomments),
     :    '(a,1x,es10.3,a,1x,f8.4,a,1x,f8.4,a)')
     :    '|  A spike of amplitude ', amp, 
     :    ' starting at ', tstart, 's'
        else
          ncomments = ncomments + 1
          write(comments(ncomments),
     :    '(a,1x,es10.3,a,1x,f8.4,a,1x,f8.4,a)')
     :    '|  A box of amplitude ', amp, ' & duration ',tstop-tstart,
     :    ' starting at ', tstart, 's'
        end if
      CASE(2)
        if (nstart .eq. nstop) then              ! step
          allocate(y(npts+2))
          y = 0.0
          do i = nstart, npts
            y(i) = amp
          end do
          ncomments = ncomments + 1
          write(comments(ncomments),
     :      '(a,1x,es10.3,a,1x,f8.4,a,1x,f8.4,a)')
     :      '|  A step of amplitude ', amp, 
     :      ' starting at ', tstart, 's'
        else
          slope = amp/float(nstop - nstart)      ! ramp
          allocate(y(npts+2))
          y = 0.0
          do i = nstart, nstop
            y(i) = slope * (i - nstart)
          end do
          do i = nstop + 1, npts
            y(i) = y(nstop)
          end do
          ncomments = ncomments + 1
          write(comments(ncomments),
     :      '(a,1x,es10.3,a,1x,f8.4,a,1x,f8.4,a)')
     :      '|  A ramp of amplitude ', amp, ' & duration ',tstop-tstart,
     :      ' starting at ', tstart, 's'
        end if
      CASE(3)
          ndelay = nstart - phase_strt*period*sps
          allocate(y(npts+2))
          y = 0.0
          do i = nstart, nstop
            y(i) = amp * sin(twopi*freq*float(i - ndelay)/sps)
          end do
          ncomments = ncomments + 1
          write(comments(ncomments),
     :      '(a,1x,es10.3,a,1x,f6.2,a,1x,f8.4,a)')
     :      '|  A sinusoid of amplitude ',
     :        amp, ', ',
     :        ancycles, 
     :      ' cycles, and period ', 
     :        period, ' s'
          ncomments = ncomments + 1
          write(comments(ncomments),
     :      '(a,1x,f4.2, a,1x,f8.4,a)')
     :      '|  beginning at ', 
     :        phase_strt, 
     :      ' of a cycle, with the motion starting at ', 
     :        tstart, 's'
      CASE(4)
        idum = 0  ! Need a line of code?
      CASE DEFAULT
        write(*,'(a,1x,i5,a)') 
     :      '  ERROR!!!, i_type = ', i_type,
     :      ' .ne. 1, 2, 3, or 4, QUITTING!'
        stop
      END SELECT
        
      SELECT CASE(i_type)
      CASE(4)
 ! Initialize seed:
        iseed = -abs(seed)

        ncomments = ncomments + 1
        if (iran_type .eq. 0) then
          write(comments(ncomments), '(a,1x,f9.4)')
     :        '|  A Gaussian noise gram: starting seed of sequence = ',       
     :        seed 
        else
          write(comments(ncomments), '(a,1x,f9.4)')
     :        '|  A uniform noise gram: starting seed of sequence = ',       
     :        seed 
        end if
        ncomments = ncomments + 1
        write(comments(ncomments),
     :        '(a,1x,f9.4,3(a,1x,f5.1),a,1x,f8.4)')
     :        '|  amp= ', amp, 
     :        ' tstart= ', tstart, 
     :        ' tstop= ', tstop, 
     :        ' tlength= ', tlength,
     :        ' sps = ', sps
        ncomments = ncomments + 1
        comments(ncomments) = '|'

        int_head(16) = ncomments

        write(*,'(a)') ' int_head(16), int_head(17), real_head(2) = '
        write(*,*)       int_head(16), int_head(17), real_head(2)
  

        loop over n_ts: DO its = 1, n_ts

! Initialize array:
!
          allocate(y(npts+2))
          
          do i = 1, npts
            y(i)=0.0
          end do

! Generate white noise with zero mean, unit variance, and proper distribution:

          if (iran_type .eq. 0) then
            do i = nstart, nstop
              y(i) = gasdev(iseed)
            end do
          else
            do i = nstart, nstop
              y(i) = 2.0 * ran1(iseed) - 1.0
            end do
          end if

          iskip = 1
          if (iskip .eq. 0) then
            sum = 0.0
            do i = nstart, nstop
              sum = sum + y(i)
            end do
            sum = sum/float(nstop-nstart+1)
            do i = nstart, nstop
              y(i) = y(i) - sum
            end do
          end if

! Scale amplitude
          call mnmaxidx(y,nstart,nstop,1,
     :                    ymin,ymax,indx_min,indx_max) 
          amax = abs(ymin)
          if(abs(ymax) .gt. abs(ymin)) then
            amax = abs(ymax)
          end if

          do i = nstart, nstop
            y(i) = amp*y(i)/amax
          end do

          c_count = ' '
          write(c_count(1:3), '(i3.3)') its
          f_smc = ' '
          f_smc = stem(1:nc_stem)//c_count//'.smc'
          call trim_c(f_smc, nc_f_smc)
          write(*,'(a)') ' f_smc = '//f_smc(1:nc_f_smc)
                     
          itype = 0
          call smcwrite(f_smc(1:nc_f_smc), precision,
     :                y, char_head, itype, 
     :                    int_head, real_head, comments)

          deallocate (y)
          
        END DO    loop over n_ts
        stop
      CASE DEFAULT
        ncomments = ncomments + 1
        comments(ncomments) = '|'

        int_head(16) = ncomments

        write(*,'(a)') ' int_head(16), int_head(17), real_head(2) = '
        write(*,*)       int_head(16), int_head(17), real_head(2)

        itype = 0
        f_smc = ' '
        f_smc = stem(1:nc_stem)//'.smc'
        call trim_c(f_smc, nc_f_smc)

        call smcwrite(f_smc(1:nc_f_smc), precision,
     :                y, char_head, itype, 
     :                    int_head, real_head, comments)

        deallocate (y)
        stop
      END SELECT

      end




      include '\forprogs\skipcmnt.for'
      include '\forprogs\distaz.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\upstr.for'
      include '\forprogs\ran1.for'
      include '\forprogs\gasdev.for'

