* ---------------------------------------------------------------- USCV2GS
      Program USCV2GS

* Convert USC V2 files to USGS SMC format

* Assumes if t_ts_per_chan = 1 that the time series is accel
* Assumes if t_ts_per_chan = 2 that the time series is accel & disp
* Assumes if t_ts_per_chan = 3 that the time series is accel, disp, & vel

* Sample control file:
*!Control file for program USCV2GS
*! Note:  may have to run files through UNIX2PC first to get into proper
*! PC format (with CR and LF characters).  Also, make sure that the
*! properties of the files are not Read Only
*!Quake_Code
* 355
*!Quake_Name
* Northridge mainshock
*!Eq Hypocenter:
* 34.209, -118.541
*!Moment Magnitude:
* 6.7
*!Include sta_code (0 = no, 1 = yes, following name, 2 = yes, from file):
* 2
*!File with Sta_code --USC station number association (only if preceeding is "2")
* usc_sta.prn
*!Number of time series per channel (e.g., 3 for A,V,D):
* 3
*!Station Filenames:
*v2x0100.dat
*v2x0401.dat
* STOP

* Dates: 12/22/00 - Written by D. Boore, patterned after CDMGV2GS
*        12/29/00 - Add dynamic allocation for array ts
*        05/16/01 - Add debug write statements, some other changes
*        10/31/01 - Obtain default name of control file via CR
*        10/31/01 - Read headers and data using free format
*        11/01/01 - Various changes (see comments for uscv1gs.for), including 
*                   getting satcode from a table lookup 
*        11/05/01 - If cannot find ista_code, assign inull
*        11/06/01 - Get initial velocity and displacements from real headers
*                   of acceleration file and save them in rhead_smc.
*        12/15/02 - Include in control file lines for file containing 
*                   sta code association even if not used
*                   Also, check for existence of f_in and allow for arbitrary
*                   number of comment lines in control file
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats

      character precision*8
      

      character cmnt_ctl(100)*80

      character thead_smc(11)*80, comments(200)*80
      integer ihead_smc(48)
      real rhead_smc(50)

      character thead_usc(25)*80, c_ncomp*1
      integer ihead_usc(100), n_ts(3)
      integer iquake_code, istanum_usc_tbl(150), ista_code_tbl(150)
      real rhead_usc(100), dt(3)
*      real ts(100000,3)
      real ts(:,:)
      allocatable :: ts

      character f_ctl*40, f_in*40, f_out*40, endchn*2, c_stop*4
      character f_code*40, quake_name*54

      character temp_staid*5, buf*80, avd_c*3
      logical f_ctl_exist, f_in_exist

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=USCV2GS.CTL;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'uscv2gs.ctl'
        inquire(file=f_ctl, exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl,status='unknown')

      call skipcmnt(nu_ctl,cmnt_ctl,ncmnt_ctl)
      read(nu_ctl,*) iquake_code

      call skipcmnt(nu_ctl,cmnt_ctl,ncmnt_ctl)
      quake_name = ' '
      read(nu_ctl, '(a)') quake_name
      call trim_c(quake_name,nc_qn)
      write(*,'(a)') ' quake_name: '//quake_name(1:nc_qn)

      call skipcmnt(nu_ctl,cmnt_ctl,ncmnt_ctl)
      read(nu_ctl, *) eqlat, eqlong
      write(*,'(a,2(1x,f10.5))') ' eqlat, eqlong: ', eqlat, eqlong

      call skipcmnt(nu_ctl,cmnt_ctl,ncmnt_ctl)
      read(nu_ctl, *) amommag
      write(*,'(a,2(1x,f10.5))') ' amommag: ', amommag

      call skipcmnt(nu_ctl,cmnt_ctl,ncmnt_ctl)
      read(nu_ctl,*) i_get_sta_code

      call skipcmnt(nu_ctl,cmnt_ctl,ncmnt_ctl)
      f_code = ' '
      read(nu_ctl, '(a)') f_code
      if (i_get_sta_code .eq. 2) then    
         call get_lun(nu_code)
         open(unit=nu_code,file=f_code,status='unknown')
         call skip(nu_code,2)
         n_code = 0
5432     continue
         read(nu_code,'(t7,i4, t67,i3)',end=2345) 
     :      ista_code_tbl(n_code+1), istanum_usc_tbl(n_code+1)
         n_code = n_code + 1
         go to 5432
2345     continue
         close(nu_code)
      end if

      call skipcmnt(nu_ctl,cmnt_ctl,ncmnt_ctl)
      read(nu_ctl,*) n_ts_per_chan

      call skipcmnt(nu_ctl,cmnt_ctl,ncmnt_ctl)
     
1234  continue

      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      write(*,'(a)') ' buf = '//buf(1:nc_buf)
      c_stop = ' '
      c_stop = buf(1:4)
      call upstr(c_stop)
      if(c_stop.eq.'STOP') then
        close(nu_ctl)
        stop
      end if

      f_in = ' '
      call RCC (1,buf,nc_buf,f_in,nc_f_in,ibgn,iend)
      call trim_c(f_in, nc_f_in)
      f_in_exist = .false.
      inquire(file=f_in(1:nc_f_in), exist=f_in_exist)
      if (.not. f_in_exist) then
        write(*,'(a)') ' ******* FILE '//f_in(1:nc_f_in)//
     :              ' DOES NOT EXIST; MOVING TO NEXT FILE ******* '
        go to 1234
      end if
      write(*,'(2a)') '  Reformatting file ', f_in(1:nc_f_in)

      write(*,'(a,1x,i3)') ' Before if, i_get_sta_code = ', 
     :  i_get_sta_code
      if (i_get_sta_code .eq. 1) then
        call RCI (2,buf,nc_buf,ista_code,ibgn,iend,ISTAT)
      else
        ista_code = 0
      end if
      write(*,'(a,1x,i3)') ' After if, ista_code = ', 
     :  ista_code

      call get_lun(nu_in)
      open(unit=nu_in,file=f_in,status='unknown')

* First get n_ts for array allocation:

* Read text headers:
      nhead_usc = 25
      do i = 1, nhead_usc
        thead_usc(i) = ' '
        read(nu_in,'(a)', end=666) thead_usc(i)
      end do

* Read integer headers:
      read(nu_in, *) (ihead_usc(i), i=1,100)
*      read(nu_in,'(16i5)') (ihead_usc(i), i=1,100)

* Close the file:
      close(nu_in)

* Get n_ts_max (= n_ts for first trace acc, increased by 1.5)

      n_ts_max = 1.5*float(ihead_usc(53))

* Allocate array:
      allocate (ts(n_ts_max,3))

* Now open the file and extract the data:

      open(unit=nu_in,file=f_in,status='unknown')

* Loop over channels:
  
      nchn = 0

2000  continue

* Read text headers:
      nhead_usc = 25
      do i = 1, nhead_usc
        thead_usc(i) = ' '
        read(nu_in,'(a)', end=666) thead_usc(i)
      end do

      nchn = nchn + 1

* Check for more than 3 channels:
      if (nchn .gt. 3) then
         write(*,'(a,1x,i5,a)') 
     :       ' !!! nchn = ', nchn, ', > 3; QUITTING !!!'
      end if

* Read integer headers:
      read(nu_in,*) (ihead_usc(i), i=1,100)
*      read(nu_in,'(16i5)') (ihead_usc(i), i=1,100)

* Read real headers:

      read(nu_in, *) (rhead_usc(i), i=1,100)
*      read(nu_in, '(8f10.3)') (rhead_usc(i), i=1,100)

* Read time series:

*      n_ts(1) = ihead_usc(53)       ! should be acceleration
*      n_ts(2) = ihead_usc(64)       ! should be velocity
*      n_ts(3) = ihead_usc(66)       ! should be displacement

* Loop over number of time series per channel
      write(*,'(a,1x,i3)') ' n_ts_per_chan = ', n_ts_per_chan
      do i_ts = 1, n_ts_per_chan
        buf = ' '
        read(nu_in,'(a)') buf
*        write(*,'(1x,a)') buf
* Extract npts, dt:
        call RCI (1,buf,80,n_ts(i_ts),ibgn,iend, ISTAT)
        call RCC (4,buf,80,avd_c,NC,ibgn,iend)
        call RCF (9,buf,80,dt(i_ts),ibgn,iend, ISTAT)
        write(*,'(1x,a,2(1x,i5), 1p1x,e10.3)') 
     :    ' i_ts, n_ts(i_ts), dt(i_ts) = ',
     :      i_ts, n_ts(i_ts), dt(i_ts)

        write(*,'(1x,a)') ' Before read ts'
        read(nu_in,*) (ts(j,i_ts), j = 1, n_ts(i_ts))
*        read(nu_in,'(8f10.3)') (ts(j,i_ts), j = 1, n_ts(i_ts))
        write(*,'(1x,a)') ' After read ts'
      end do

* Correct acceleration to g:   ! NO-- Units are in cm/s/s
*      do j = 1, n_ts(1)
*        ts(j,1) = rhead_usc(5) * ts(j,1)
*      end do

* Set up text headers:
      do i = 1, 11
        thead_smc(i) = ' '
      end do
      thead_smc(2) = '*'
      thead_smc(3) = thead_usc(6)(13:17) ! Not official SMC format
      thead_smc(4)(1:25) = thead_usc(4)(1:25) ! Not official SMC format
      thead_smc(4)(27:66) = thead_usc(3)(1:40) ! Not official SMC format
      thead_smc(5) = 
     :    'Moment Mag=          Ms=          Ml='
      write(thead_smc(5)(12:20),'(f9.2)') amommag
      thead_smc(6) = 
     :    'station =                                component='
      thead_smc(6)(11:40) = thead_usc(7)(1:30)

      if (ihead_usc(27) .gt. 360) then      ! assume vertical
        thead_smc(6)(56:57) = 'UP'
      else
        write(thead_smc(6)(53:57),'(i5)') ihead_usc(27)
      end if
      thead_smc(7) = 
     :    'epicentral dist ='
* Extract station coordinates and compute hypocentral distance:
*      stalat = real_head(9)
*      stalong = real_head(10)

* A more accurate station location is given using d, m, s, assuming that
* s is given accurately.

      alat_d = ihead_usc(10)
      alat_m = ihead_usc(11)
      alat_s = ihead_usc(12)
      stalat = alat_d + (alat_m + alat_s/60.0)/60.0
      along_d = ihead_usc(13)
      along_m = ihead_usc(14)
      along_s = ihead_usc(15)
      stalong = along_d + (along_m + along_s/60.0)/60.0

      if (stalong .gt. 0.0) stalong = -stalong   ! correct sign of longitude

* Here is more general code:
*      read(thead_usc(6)(23:28),'(f6.3)') stalat
*      read(thead_usc(6)(34:40),'(f7.3)') stalong
*      c_lat = thead_usc(6)(30:30)
*      c_long = thead_usc(6)(42:42)
*      call upstr(c_lat)
*      if (c_lat .eq. 'S') stalat = -abs(stalat)
*      call upstr(c_long)
*      if (c_long .eq. 'W') stalong = -abs(stalong)

      call distaz( -1.0, eqlat,eqlong,stalat,stalong,
     :    rdeg, rkm, az, baz)
      write(thead_smc(7)(18:26),'(f9.1)') rkm
      thead_smc(8) = 
     :    'inst type=           data source = USC'
               
*      temp_c = ' '
*      temp_c = thead_usc(6)(40:47) 
*      call trim_c(temp_c,nc_tc)
*      if (nc_tc .gt. 5) nc_tc = 5
*      thead_smc(8)(11:11+nc_tc-1) = temp_c(1:nc_tc)

      do i = 9, 11
        thead_smc(i) = '*'
      end do

* Set up comments:
      comments(1) = '|'
      comments(2) = '| USC file '//f_in
      comments(3) = '| Reformatted to SMC format, using USCV2GS'
      comments(4) = '|'
      comments(5) = '| Here follow the USC text headers:'
      comments(6) = '|'
      ncomments = 6
* Now add USC text headers as comments:
      do i = 1, nhead_usc
        comments(i+ncomments) = '| '//thead_usc(i)(1:78)
      end do
      ncomments = ncomments + nhead_usc

* Set up integer headers independent of type of motion:
      inull = -32768
      do i = 1, 48
        ihead_smc(i) = inull
      end do

      read(thead_usc(6)(54:57),'(i4)') ihead_smc(8)

      ihead_smc(9) = nchn

      if (ihead_usc(27) .gt. 360) then      ! assume vertical
        ihead_smc(13) = 0
        ihead_smc(14) = 0
      else
        ihead_smc(13) = 90
        ihead_smc(14) = ihead_usc(27)
      end if

      ihead_smc(16) = ncomments

      temp_staid = ' '
      temp_staid = thead_usc(6)(13:17)
      call notnumrc(temp_staid, 5, iflag)
      if (iflag .eq. 0) then
        read(temp_staid,'(i5)') ihead_smc(30)
      end if

      ihead_smc(31) = 1

      ihead_smc(40) = iquake_code

      if(i_get_sta_code .eq. 2) then  ! look up code
        read(thead_usc(6),'(t15,i3)') istanum_usc 
        do i = 1, n_code
           if (istanum_usc .eq. istanum_usc_tbl(i)) then
             ista_code = ista_code_tbl(i)
             go to 3377
           end if
        end do
        ista_code = inull
3377    continue
      end if

      ihead_smc(30) = istanum_usc
      ihead_smc(41) = ista_code

* Set up real headers independent of type of motion:
      do i = 1, 50
        rhead_smc(i) = 1.7e+38
      end do

*      rhead_smc(2) = 1.0/rhead_usc(61) ! this assumes that dt is same for a,v,d
      rhead_smc(3) = eqlat
      rhead_smc(4) = eqlong
      rhead_smc(6) = amommag
      rhead_smc(11) = stalat
      rhead_smc(12) = stalong
      rhead_smc(17) = rkm
      rhead_smc(18) = az
      rhead_smc(22) = 1.0/rhead_usc(1)  ! natural frequency
      rhead_smc(23) = rhead_usc(2)      ! damping
      rhead_smc(24) = rhead_usc(6)      ! sensitivity
      rhead_smc(33) = rhead_usc(62) - rhead_usc(63) ! lc start
      rhead_smc(34) = rhead_usc(62)                 ! lc stop
      rhead_smc(35) = rhead_usc(58) - rhead_usc(59) ! hc start
      rhead_smc(36) = rhead_usc(58)                 ! hc stop
      rhead_smc(38) = rhead_usc(71)                 ! initial velocity
      rhead_smc(39) = rhead_usc(76)                 ! initial displacement

* Now do motion-specific things (define filenames, using  v2s scheme
* .as1, .as2, .as3, etc; fill rhead(29),..,rhead(32)
* with peak values and times, fill ihead(17) with number of time series 
* values

      do i_ts = 1, n_ts_per_chan

        ihead_smc(17) = n_ts(i_ts)
        ihead_smc(32) = n_ts(i_ts)
      
        rhead_smc(2) = 1.0/dt(i_ts)

* Get peak motions:
      
        call mnmaxidx(ts(1,i_ts), 1, ihead_smc(17), 1,
     :             ymin, ymax, indx_min, indx_max) 
        tmin = float(indx_min-1)/rhead_smc(2)
        tmax = float(indx_max-1)/rhead_smc(2)
        rhead_smc(29) = tmax
        rhead_smc(30) = ymax
        rhead_smc(31) = tmin
        rhead_smc(32) = ymin

        f_out = ' '
        nstem = index(f_in(1:nc_f_in),'.') - 1
        write(c_ncomp,'(i1)') nchn
        if( i_ts .eq. 1) then
          f_out = f_in(1:nstem)//'.as'//c_ncomp
          itype = 2
        else if ( i_ts .eq. 2) then
          f_out = f_in(1:nstem)//'.vs'//c_ncomp
          itype = 3
        else
          f_out = f_in(1:nstem)//'.ds'//c_ncomp
          itype = 4
        end if
      precision = ' '
      precision = 'STANDARD'
      ihead_smc(47) = ihead_smc(1)

        call SMCWrite(f_out, precision,
     :              ts(1,i_ts), thead_smc, itype, 
     :                    ihead_smc, rhead_smc, comments)

      end do

*  check for end-of-channel marker:
      buf = ' '
      read(nu_in, '(a)') buf
      endchn = ' '
      call RCC (1,buf,80,endchn,NC,ibgn,iend)

      if (endchn(1:2) .ne. '/&') then
         write(*,'(a)') ' Problem with endchn, ne /&, quitting'
         stop
      end if

*  go back for a new channel:

      go to 2000

666   continue
      close(nu_in)

      deallocate (ts)


      go to 1234

      end
* ---------------------------------------------------------------- USCV2GS

      include '\forprogs\notnumrc.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\upstr.for'
      include '\forprogs\skip.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\distaz.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\rc_subs.for'
      include '\forprogs\skipcmnt.for'

          